@@ -1,6 +1,6 @@
-mod analytic_gaussian;
 mod dp_parameters;
 mod noise_aggregator;
+mod noise_parameters;
 mod noisy_count_threshold;
 mod percentile;
 mod stats_error;
@@ -9,7 +9,6 @@ mod typedefs;
 #[cfg(feature = "pyo3")]
 mod register_pyo3;
 
-pub use analytic_gaussian::*;
 pub use dp_parameters::*;
 pub use noise_aggregator::*;
 pub use noisy_count_threshold::*;
